# initialize all necessary libraries
library(cluster)
library(psych)

# Replace the path where you see file with your path
path="~/Dropbox/MST_COURSES/SS16/R/"

# read CSV file - (kmeans_data.csv) - convert to a matrix
data1 <- read.table(file=paste0(path,'kmeans_data.csv'), sep=',', header=T, row.names=1)
str(data1)
summary(data1)

#Convert to matrix format
data.p <- as.matrix(data1)

#Convert data from counts to percents
data.p <- prop.table(data.p,1)*100

#Provide Z-score standardize data?
kdata <- na.omit(data.p) 
kdata <- scale(kdata)

n.lev = 15

# Calculate the within groups sum of squared error (SSE) for the number of cluster solutions selected by the user
wss <- rnorm(10)
while (prod(wss==sort(wss,decreasing=T))==0) {
  wss <- (nrow(kdata)-1)*sum(apply(kdata,2,var))
  for (i in 2:n.lev) wss[i] <- sum(kmeans(kdata, centers=i)$withinss)}

# Plot within groups SSE against all tested cluster solutions for actual data
# 1st: Log scale, 2nd: Normal scale
xrange <- range(1:n.lev)
yrange <- range(min(log(wss)),max(log(wss)))
plot(xrange,yrange, type='n', xlab='Cluster Solution', ylab='Log of Within Group SSE', main='Cluster Solutions against Log of SSE')
lines(log(wss), type="b", col='blue')

yrange <- range(min(wss),max(wss))
plot(xrange,yrange, type='n', xlab="Cluster Solution", ylab="Within Groups SSE", main="Cluster Solutions against SSE")
lines(wss, type="b", col='blue')

# Ask for user input - Select the appropriate number of clusters
choose.clust <- function(){readline("What clustering solution would you like to use? ")} 
clust.level <- as.integer(choose.clust())

# Apply K-means cluster solutions - append clusters to CSV file
fit5 <- kmeans(kdata, clust.level)
aggregate(kdata, by=list(fit5$cluster), FUN=mean)
clust.out <- fit5$cluster
kclust <- as.matrix(clust.out)
kclust.out <- cbind(kclust, data1)
write.table(kclust.out, file=paste0(path,"kmeans_out_5.csv"), sep=",")

# FURTHER ANALYSIS WITH PCA
# Display Principal Components plot of data with clusters identified
clusplot(kdata, fit5$cluster, shade=F, labels=2, lines=0, color=T, lty=4, main='Principal Components plot showing K-means clusters')

# Send output to files
kclust.out.p <- prop.table(as.matrix(kclust.out),1)*100
out <- capture.output(describeBy(kclust.out.p,kclust))
cat(out,file=paste0(path,"Kmeans_out.txt"), sep='\n', append=F)

#repeat process for other K
choose.clust <- function(){readline("What clustering solution would you like to use? ")} 
clust.level <- as.integer(choose.clust())

fit6 <- kmeans(kdata, clust.level)
aggregate(kdata, by=list(fit6$cluster), FUN=mean)
clust.out <- fit6$cluster
kclust <- as.matrix(clust.out)
kclust.out <- cbind(kclust, data1)
write.table(kclust.out, file=paste0(path,"kmeans_out_6.csv"), sep=",")

clusplot(kdata, fit6$cluster, shade=F, labels=2, lines=0, color=T, lty=4, main='Principal Components plot showing K-means clusters')
# end of script

