# Attitude dataset
# Load necessary libraries
library(datasets)

# Inspect data structure
str(attitude)
# What kind of data does the dataset contain? Numeric? Text? Ordinal? Categorical? Binary?
# How many different data points do we have and how many attributes?

# Summarise data
summary(attitude)
# Do you observe any outliers? Any huge variations in the values?

# Subset the attitude data
#dat = attitude[,c(3,4)]
dat=attitude

# Plot subset data
plot(dat, main = "Exploratory plot", pch =20, cex =2)

# Perform K-Means with 2 clusters
set.seed(7)
km1 = kmeans(dat, 2, nstart=100)

km1
#Observe the outcome of algorithm
#What is the size of the clusters created?
#What are the reference vectors for each cluster?
#What is the SSE?

# Plot results
plot(dat, col =(km1$cluster +1) , main="K-Means result with 2 clusters", pch=20, cex=2)
#Observe the results

# Check for the optimal number of clusters given the data
mydata <- dat
wss <- (nrow(mydata)-1)*sum(apply(mydata,2,var))
for (i in 2:15)
  wss[i] <- sum(kmeans(mydata,centers=i)$withinss)
  plot(1:15, wss, type="b", xlab="Number of Clusters",
       ylab="Within groups sum of squares",
       main="Assessing the Optimal Number of Clusters with the Elbow Method",
       pch=20, cex=2)
#Which could be the optimal number of clusters?
  
# Perform K-Means with the optimal number of clusters identified from the Elbow method
set.seed(7)
km2 = kmeans(dat, 4, nstart=100)

# Examine the result of the clustering algorithm
km2
plot(dat, col =(km2$cluster +1) , main="K-Means result with 4 clusters", pch=20, cex=2)

#Project the result using the first two components and perform clustering
clusplot(dat, km2$cluster, color=TRUE, shade=TRUE, labels=2, lines=0)

#Computer measures for clustering
#library(fpc)
#cluster.stats(d, km2$cluster, km1$cluster)

library(pvclust)
fit <- pvclust(t(dat), method.hclust="ward.D2",method.dist="euclidean")
plot(fit) # dendogram with p values
# add rectangles around groups highly supported by the data
pvrect(fit, alpha=.95)

#fit2 = agnes(x=dat, diss = FALSE, stand = TRUE, method = "average")
#plot(fit2)