# pima indians analysis
# initialize all necessary libraries
library(cluster)
library(psych)

path="~/Dropbox/MST_COURSES/SS16/R/"

# comma delimited data and no header for each variable
RawData <- read.table(paste0(path,"pima-indians-diabetes.data"),sep = ",",header=FALSE)

# In RawData, the response variable is its last column;
#and the remaining columns are the predictor variables.

responseY <- RawData[,dim(RawData)[2]]
predictorX <- RawData[,1:(dim(RawData)[2]-1)]

# For the convenience of visualization, we take the first two principle components as the new feature
# variables and conduct k-means only on these two dimensional data.
pca <- princomp(predictorX, cor=T) # principal components analysis using correlation matrix
pc.comp <- pca$scores
pc.comp1 <- -1*pc.comp[,1] # principal component 1 scores (negated for convenience)
pc.comp2 <- -1*pc.comp[,2] # principal component 2 scores (negated for convenience)

#visualizing
X <- cbind(pc.comp1, pc.comp2)
cl <- kmeans(X,13)
cl$cluster
plot(pc.comp1, pc.comp2,col=cl$cluster)
points(cl$centers, pch=16)

#HAC results
#In R, library cluster implements hierarchical clustering using the agglomerative nesting algorithm (agnes).
#The first argument x in agnes specifies the input data matrix or the dissimilarity matrix, depending on
#the value of the diss argument. If diss=TRUE, x is assumed to be a dissimilarity matrix. If diss=FALSE,
#x is treated as a matrix of observations. The argument stand = TRUE indicates that the data matrix is
#standardized before calculating the dissimilarities.

#Each variable (a column in the data matrix) is standardized by first subtracting the mean value of the
#variable and then dividing the result by the mean absolute deviation of the variable. If x is already a
#dissimilarity matrix, this argument will be ignored.

#To merge two clusters into a new cluster, the argument method specifies the measurement of
#between-cluster distance. method="single" is for single linkage clustering, method="complete"
#for complete linkage clustering, and method="average" for average linkage clustering.
#The default is method="average".

#For clarity of illustration, we use only the first 25 observations to run the agglomerative nesting
#algorithm (agnes). The function as.dendrogram generates a dendrogram using as input the agglomerative
#clustering result obtained by agnes.

dataset=predictorX[1:100,]

agn <- hclust(dist(dataset), method= "single")
plot(agn, hang=-1)
rect.hclust(agn, k=2, border = "red")

agn <- hclust(dist(dataset), method= "complete")
plot(agn, hang=-1)
rect.hclust(agn, k=2, border = "red")

agn <- hclust(dist(dataset), method= "average")
plot(agn, hang=-1)
rect.hclust(agn, k=2, border = "red")

agn <- hclust(dist(dataset), method= "ward.D2")
plot(agn, hang=-1)
rect.hclust(agn, k=2, border = "red")
