#NMF2
#install.packages("rnmf_0.5.tar.gz", repos = NULL, type = "source")
library(rNMF)

#First load the build-in data set "Symbols_c", a 5625 by 30 matrix where each column contains a
#vectorized 75 by 75 image.
data(Symbols_c)

see(Symbols_c, title = "Corrupted data set")

res <- rnmf(Symbols_c, k = 4, showprogress = FALSE, my.seed = 100)

#The basis vectors of the regular NMF decomposition are given by res$W:
see(res$W, title = "Regular NMF basis", layout = c(1,4))
#Shadows in above images indicate that corruptions were not removed and contaminated the decomposition.

#The reconstruction with K=4
see(res$fit, title = "Regular NMF reconstruction with k = 4")

#Now the same data is decomposed with robust NMF (rNMF) with 3% trimming (gamma = 0.03, k = 4)
res2 <- rnmf(Symbols_c, k = 4, gamma = 0.03, showprogress = FALSE, my.seed = 100, tol = 0.0001, maxit = 50)

#rnmf reconstruction:
see(res2$fit, title = "rNMF reconstruction with k = 4")

see(res2$W, title = "rNMF basis vectors", layout = c(1,4))

#The results are more clear. The outliers are extracted as follows:
outliers <- matrix(0, nrow = nrow(Symbols_c), ncol = ncol(Symbols_c))
outliers[res2$trimmed[[res2$niter]]] <- 1
see(outliers, title = "Outliers extracted by rNMF")

####Example 2

#In this example we compare the compression of a single corrupted image by both regular NMF and rNMF.
#First we load a build-in corrupted face image.

data(face)
#Let???s look at the corrupted face image:
  
see(face, title = "Corrupted face image", col = "grey", input = "single")

#Regular NMF compression (trim = FALSE)
#A compression by the regular NMF is shown in the following:

res <- rnmf(face, k = 10, showprogress = FALSE, my.seed = 100)
see(res$fit, title = "NMF compression", col = "grey", input = "single")

#The compression is heavily contaminated.
#rNMF compression with trimming

#Now we compress the corrupted image with rNMF:

res2 <- rnmf(face, k = 10, gamma = 0.025, showprogress = FALSE, my.seed = 100)
see(res2$fit, title = "rNMF compression", col = "grey", input = "single")

#Much better, isn???Isn't it?